<?php

namespace Modules\Cotacao\Modules\Pedido\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\ORM\DateTime;
use Modules\Cotacao\Modules\Pedido\Services\Sup95000AprovacaoPedidosColetaInternetService;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class Sup95000AprovacaoPedidosColetaInternetController
{
    
    public function __construct(Sup95000AprovacaoPedidosColetaInternetService $service)
    {
        $this->service  = $service;
    }
    
    public function gerarCotacaoPrecos(DTO\Request\Row $request, DTO\Response $response)
	{
	    $row = $request->getRow();
	    
	    $dnrsemini = DateTime::createFromFormat('d/m/Y',$row['DTINICIAL']);
        $row['NRSEMINI'] = $dnrsemini->format("W");
        
        $dnrsemfin = DateTime::createFromFormat('d/m/Y',$row['DTFINAL']);
        $row['NRSEMFIN'] = $dnrsemfin->format("W");
	    
		try {
		    $this->service->beginTransaction();
			
			$listaErros = array();
            foreach ($row['PEDIDOS'] as $pedido) {
                $erro = array();
                
                $this->verificaTermoAceite($pedido['CDFORNECED'], $erro);
                $this->verificaPedidoColetaGerado($pedido, $erro);
                $this->verificaInconsistencia($pedido, $erro);
                
                $this->executaCotacaoPreco($row, $pedido, $erro);
                
                if (count($erro)) {
                	$listaErros[$pedido['NRPEDIDOCOL']] = $erro;
                }
            }
            
            $this->service->commit();
            
            $response->addDataSet(new DataSet("listaErros", $listaErros));
		} catch (\Exception $e) {
		    $this->service->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	private function verificaTermoAceite($cdforneced, &$erro)
	{
	    $termoAceite = $this->service->retornaTermoAceite($cdforneced);
	    
	    if ($termoAceite == 'S') {
	        array_push($erro, "MSG_TERMO_ACEITE");
	    }
	}
	
	private function verificaPedidoColetaGerado($pedido, &$erro)
	{
	    $existePedidoColeta = $this->service->verificaPedidoColetaGerado($pedido);
	    
	    if ($existePedidoColeta) {
	        array_push($erro, "MSG_PEDIDO_GERADO");
	    }
	}
	
	private function verificaInconsistencia($pedido, &$erro)
	{
	    $cdfilial = $pedido["CDFILIAL"];
        $nrpedidocol = $pedido["NRPEDIDOCOL"];
        $idconsiste = 'S';
	    
	    $inconsistencia = $this->service->procAceitaPedcotweb($cdfilial, $nrpedidocol, $idconsiste);
	    
	    if ($inconsistencia['P_IDACEITEOK'] === 'ERRO') {
	    	array_push($erro, "MSG_INCONSISTENTE");
	    }
	}
	
	private function executaCotacaoPreco($row, $pedido, &$erro)
	{
	    if (!$erro) {
    	    $pedido['NRSEMINI'] = $row['NRSEMINI'];
            $pedido['NRSEMFIN'] = $row['NRSEMFIN'];
            $pedido['ENTRALISTA'] = $row['ENTRALISTA'];
            
            $retorno = $this->service->gerarCotacaoPrecos($pedido);
            
            if ($retorno['P_COTA_N_GER'] == 'S') {
                array_push($erro, "MSG_NAO_GERADO");
            }
	    }
	}
	
}